/**
Filename: GUD900_Serial_Async.h
Purpose: Establish a communication interface between an
		 Arduino Uno board and a Noritake GUD display 
		 module through Asynchronous Serial communication. 
		 This interface features both read and write functionality.
Author: Cody Johnson
Email: cody.ele@noritake.com
Note: SoftwareSerial.h was heavily referenced while
	  writing this interface.
*/

#ifndef GUD900_SERIAL_ASYNC_H
#define GUD900_SERIAL_ASYNC_H

#include <inttypes.h>
#include <Noritake_VFD_GUD900.h>

#define _SS_MAX_RX_BUFF 64 // RX buffer size

class GUD900_Serial_Async : public GUD900_Interface
{
	private:
		uint8_t _transmitBitMask;
		uint8_t _receiveBitMask;
		uint8_t _pcint_maskvalue;
		
		uint16_t _rx_delay_centering;
		uint16_t _rx_delay_intrabit;
		uint16_t _rx_delay_stopbit;
		uint16_t _tx_delay;
		uint16_t _inverse_logic:1;
		uint16_t _buffer_overflow:1;
		
		volatile uint8_t *_receivePortRegister;
		volatile uint8_t *_transmitPortRegister;
		volatile uint8_t *_pcint_maskreg;
			
		static char _receive_buffer[_SS_MAX_RX_BUFF];
		static volatile uint8_t _receive_buffer_tail;
		static volatile uint8_t _receive_buffer_head;
		static GUD900_Serial_Async *active_object;
		
		static uint16_t subtract_cap(uint16_t num, uint16_t sub);
		static inline void tunedDelay(uint16_t delay);
		
		void recv() __attribute__((__always_inline__));
		void setRxIntMsk(bool enable);
		void tx_pin_write(uint8_t pin_state);
		uint8_t rx_pin_read();
		
	protected:
		//Pin definitions
		unsigned OUT_PIN:4;
		unsigned BUSY_PIN:4;
		unsigned RESET_PIN:4;
		unsigned DEBUG_PIN:4;
		unsigned IN_PIN:4;
	
	public:
		GUD900_Serial_Async(long baudRate, uint8_t out, uint8_t in, uint8_t busy, uint8_t reset);
		void init();
		void write(uint8_t data);
		int read();
		int available();
		bool isListening();
		bool listen();
		bool stopListening();
		void hardReset();
		
		operator bool() {return true;}

		static inline void handle_interrupt() __attribute__((__always_inline__));
};

// Arduino 0012 workaround
#undef int
#undef char
#undef long
#undef byte
#undef float
#undef abs
#undef round

#endif