/*
Filename: TempDemoAsync.ino
Purpose: Demonstrate the GU-D touch switch capabilities as
		 well as multiple custom defined windows and
		 different font sizes.
Author: Cody Johnson
Email: cody.ele@noritake.com
Note: This demo was specifically made for the 
	  GU128X32D-D903S module.
*/

#include <GUD900_Interface.h>
#include <GUD900_Serial_Async.h>
#include <GUD900_Serial_SPI.h>
#include <GUD900_TWI.h>
#include <Noritake_VFD_GUD900.h>

#define TOUCH_STATUS_IDENTIFIER 17

GUD900_Serial_Async interface(38400, 3, 4, 5, 7); //(BAUD, OUT, IN, BUSY, RESET) NOTE: Tie PIN7 to GND to enable interrupts and read functionality

Noritake_VFD_GUD900 vfd;

char valueInt[4];
int temp = 300;
int placeValue = 1;

uint8_t value;
uint8_t touchData;

int switchNum = 0;
int statusBit = 0;
int count = 0;
int touchedOne = 0;
int touchedTwo = 0;
int touchedThree = 0;
int touchedFour = 0;
int touchedHold = 0;
int holdFlag = 0;
uint8_t actMemory[16];

/* 27x16 image "upArrow2" */
static const uint8_t PROGMEM image_upArrow[] = {
	0xff,0xff,0x80,0x01,0x80,0x01,0x80,0x01,0x80,0x01,0x80,0x19,0x80,0x39,0x80,0x79,
	0x80,0xf9,0x81,0xf9,0x83,0xf9,0x87,0xf9,0x8f,0xf9,0x9f,0xf9,0x8f,0xf9,0x87,0xf9,
	0x83,0xf9,0x81,0xf9,0x80,0xf9,0x80,0x79,0x80,0x39,0x80,0x19,0x80,0x01,0x80,0x01,
	0x80,0x01,0x80,0x01,0xff,0xff
};

/* 27x16 image "downArrow" */
static const uint8_t PROGMEM image_downArrow[] = {
	0xff,0xff,0x80,0x01,0x80,0x01,0x80,0x01,0x80,0x01,0x98,0x01,0x9c,0x01,0x9e,0x01,
	0x9f,0x01,0x9f,0x81,0x9f,0xc1,0x9f,0xe1,0x9f,0xf1,0x9f,0xf9,0x9f,0xf1,0x9f,0xe1,
	0x9f,0xc1,0x9f,0x81,0x9f,0x01,0x9e,0x01,0x9c,0x01,0x98,0x01,0x80,0x01,0x80,0x01,
	0x80,0x01,0x80,0x01,0xff,0xff
};

/* 15x32 image "leftArrow" */
static const uint8_t PROGMEM image_leftArrow[] = {
	0xff,0xff,0xff,0xff,0x80,0x00,0x00,0x01,0x80,0x01,0x00,0x01,0x80,0x03,0x80,0x01,
	0x80,0x07,0xc0,0x01,0x80,0x0f,0xe0,0x01,0x80,0x1f,0xf0,0x01,0x80,0x3f,0xf8,0x01,
	0x80,0x7f,0xfc,0x01,0x80,0xff,0xfe,0x01,0x81,0xff,0xff,0x01,0x81,0xff,0xff,0x01,
	0x80,0x00,0x00,0x01,0x80,0x00,0x00,0x01,0xff,0xff,0xff,0xff
};

/* 15x32 image "rightArrow" */
static const uint8_t PROGMEM image_rightArrow[] = {
	0xff,0xff,0xff,0xff,0x80,0x00,0x00,0x01,0x80,0x00,0x00,0x01,0x81,0xff,0xff,0x01,
	0x81,0xff,0xff,0x01,0x80,0xff,0xfe,0x01,0x80,0x7f,0xfc,0x01,0x80,0x3f,0xf8,0x01,
	0x80,0x1f,0xf0,0x01,0x80,0x0f,0xe0,0x01,0x80,0x07,0xc0,0x01,0x80,0x03,0x80,0x01,
	0x80,0x01,0x00,0x01,0x80,0x00,0x00,0x01,0xff,0xff,0xff,0xff
};

/* 32x16 image "holdButton" */
static const uint8_t PROGMEM image_holdButton[] = {
	0xff,0xff,0x80,0x01,0x80,0x01,0x80,0x01,0x80,0x01,0x80,0x01,0x87,0xf1,0x80,0x81,
	0x80,0x81,0x87,0xf1,0x80,0x01,0x83,0xe1,0x84,0x11,0x84,0x11,0x84,0x11,0x83,0xe1,
	0x80,0x01,0x87,0xf1,0x80,0x11,0x80,0x11,0x80,0x11,0x80,0x01,0x87,0xf1,0x84,0x11,
	0x84,0x11,0x83,0xe1,0x80,0x01,0x80,0x01,0x80,0x01,0x80,0x01,0x80,0x01,0xff,0xff
};

void setup()
{
	Serial.begin(9600);
	_delay_ms(500);
	vfd.begin(128, 32);
	vfd.interface(interface);
	vfd.isModelClass(903);
	vfd.isGeneration('S');
	vfd.GUD900_reset();
	vfd.GUD900_init();
	
	vfd.println("Oven");
	vfd.print("Temp");
	vfd.GUD900_setCursor(33, 0);
	vfd.GUD900_drawImage_p(32, 16, image_holdButton);

	printTemp(temp);
	
	vfd.GUD900_defineWindow(1, 70, 0, 27, 16);
	vfd.GUD900_defineWindow(2, 70, 17, 27, 16);
	vfd.GUD900_defineWindow(3, 98, 0, 15, 32);
	vfd.GUD900_defineWindow(4, 113, 0, 15, 32);
	
	drawButtons();
	
	vfd.GUD900_touch_set(0x02);
}

void loop()
{
	while(interface.available() > 0){
		touchData = interface.read();
		if(touchData >= 0 && touchData <= 32){
			Serial.print("Value: ");
			Serial.println(touchData);
			if(touchData == TOUCH_STATUS_IDENTIFIER){
				count = 1;
			}else if(count == 1){
				switchNum = touchData;
				count = 2;
			}else if(count == 2){
				statusBit = touchData;
				count = 0;
			}
			touchData = 0;
		}
		if(count == 0){
			touchMemory(switchNum, statusBit);
		}
		updateButtons();
	}
}

void printTemp(int tempIn){
	String tempVal = String(tempIn);
	tempVal.toCharArray(valueInt , 4);
	vfd.GUD900_setCursor(0, 17);
	vfd.GUD900_setFontSize(2, 2, false);
	if(tempIn < 100 && tempIn >= 10){
		vfd.print("0");
	}else if(tempIn < 10){
		vfd.print("00");
	}
	vfd.print(valueInt);
	vfd.print(248);
	vfd.print("F");
}

void drawButtons(){
	vfd.GUD900_selectWindow(1);
	vfd.GUD900_drawImage_p(27, 16, image_upArrow);
	
	vfd.GUD900_selectWindow(2);
	vfd.GUD900_drawImage_p(27, 16, image_downArrow);
	
	vfd.GUD900_selectWindow(3);
	vfd.GUD900_drawImage_p(15, 32, image_leftArrow);
	
	vfd.GUD900_selectWindow(4);
	vfd.GUD900_drawImage_p(15, 32, image_rightArrow);
	vfd.GUD900_selectWindow(0);
}

void updateButtons(){
	if(statusBit == 1){
		vfd.GUD900_invertOn();
		if((actMemory[4] == 1 || actMemory[5] == 1) && touchedOne == 0 && (switchNum == 4 || switchNum == 5)){
			vfd.GUD900_selectWindow(1);
			vfd.GUD900_drawImage_p(27, 16, image_upArrow);
			if(!holdFlag){
				temp += placeValue;
			}
			touchedOne = 1;
		}else if((actMemory[12] == 1 || actMemory[13] == 1) && touchedTwo == 0 && (switchNum == 12 || switchNum == 13)){
			vfd.GUD900_selectWindow(2);
			vfd.GUD900_drawImage_p(27, 16, image_downArrow);
			if(!holdFlag){
				temp -= placeValue;
			}
			touchedTwo = 1;
		}else if((actMemory[6] == 1 || actMemory[14] == 1) && touchedThree == 0 && (switchNum == 6 || switchNum == 14)){
			vfd.GUD900_selectWindow(3);
			vfd.GUD900_drawImage_p(15, 32, image_leftArrow);
			placeValue *= 10;
			if(placeValue > 100){
				placeValue = 1;
			}
			touchedThree = 1;
		}else if((actMemory[7] == 1 || actMemory[15] == 1) && touchedFour == 0 && (switchNum == 7 || switchNum == 15)){
			vfd.GUD900_selectWindow(4);
			vfd.GUD900_drawImage_p(15, 32, image_rightArrow);
			placeValue /= 10;
			if(placeValue < 1){
				placeValue = 100;
			}
			touchedFour = 1;
		}else if((actMemory[2] == 1 || actMemory[3] == 1) && touchedHold == 0 && (switchNum == 2 || switchNum == 3)){
			if(holdFlag){
				vfd.GUD900_invertOff();
			}
			vfd.GUD900_selectWindow(0);
			vfd.GUD900_setCursor(33, 0);
			vfd.GUD900_drawImage_p(32, 16, image_holdButton);
			touchedHold = 1;
			holdFlag = ~holdFlag;
		}
		vfd.GUD900_invertOff();
	}
	if(statusBit == 0 || statusBit > 1){
		if((actMemory[4] == 0 || actMemory[5] == 0) && touchedOne == 1 && (switchNum == 4 || switchNum == 5)){
			vfd.GUD900_selectWindow(1);
			vfd.GUD900_drawImage_p(27, 16, image_upArrow);
			touchMemory(4, 0);
			touchMemory(5, 0);
			touchedOne = 0;
		}else if((actMemory[12] == 0 || actMemory[13] == 0) && touchedTwo == 1 && (switchNum == 12 || switchNum == 13)){
			vfd.GUD900_selectWindow(2);
			vfd.GUD900_drawImage_p(27, 16, image_downArrow);
			touchMemory(12, 0);
			touchMemory(13, 0);
			touchedTwo = 0;
		}else if((actMemory[6] == 0 || actMemory[14] == 0) && touchedThree == 1 && (switchNum == 6 || switchNum == 14)){
			vfd.GUD900_selectWindow(3);
			vfd.GUD900_drawImage_p(15, 32, image_leftArrow);
			touchMemory(6, 0);
			touchMemory(14, 0);
			touchedThree = 0;
		}else if((actMemory[7] == 0 || actMemory[15] == 0) && touchedFour == 1 && (switchNum == 7 || switchNum == 15)){
			vfd.GUD900_selectWindow(4);
			vfd.GUD900_drawImage_p(15, 32, image_rightArrow);
			touchMemory(7, 0);
			touchMemory(15, 0);
			touchedFour = 0;
		}else if((actMemory[2] == 0 || actMemory[3] == 0) && touchedHold == 1 && (switchNum == 2 || switchNum == 3)){
			touchMemory(2, 0);
			touchMemory(3, 0);
			touchedHold = 0;
		}
	}
	vfd.GUD900_selectWindow(0);
	if(temp < 0){
		temp = 0;
	}else if(temp > 999){
		temp = 999;
	}
	printTemp(temp);
}

void touchMemory(uint8_t switchVal, uint8_t touchValue){
	if(touchValue > 1){
		touchValue = 0;
	}
	if(switchVal > 0 && switchVal < 16){
		actMemory[switchVal] = touchValue;
	}
}
