#include <GUD900_Interface.h>
#include <GUD900_Serial_Async.h>
#include <GUD900_Serial_SPI.h>
#include <GUD900_TWI.h>
#include <Noritake_VFD_GUD900.h>

#define TOUCH_STATUS_IDENTIFIER 0x14

GUD900_TWI interface (0x50, 400000, 6, 7, 8); //(slaveAddress, clockFrequency, MBUSY, TRDY, RESET)

Noritake_VFD_GUD900 vfd;

char valueInt[4];
int temp = 300;
int placeValue = 1;

uint8_t value;
uint8_t touchData;

int switchNum = 0;
int statusBit = 0;
int count = 0;
int holdFlag = 0;
uint8_t actMemory[16];

int offset = 0;
int switch_position = 15;

uint8_t bar0[] = {
  0b00000000,
  0b00000000,
  0b00000000,
  0b00000000,
  0b00000000
};

uint8_t bar1[] = {
  0b00000010,
  0b00000010,
  0b00000010,
  0b00000010,
  0b00000010
};

uint8_t bar2[] = {
  0b00000110,
  0b00000110,
  0b00000110,
  0b00000110,
  0b00000110
};

uint8_t bar3[] = {
  0b00001110,
  0b00001110,
  0b00001110,
  0b00001110,
  0b00001110
};

uint8_t bar4[] = {
  0b00011110,
  0b00011110,
  0b00011110,
  0b00011110,
  0b00011110
};

uint8_t bar5[] = {
  0b00111110,
  0b00111110,
  0b00111110,
  0b00111110,
  0b00111110
};

uint8_t bar6[] = {
  0b01111110,
  0b01111110,
  0b01111110,
  0b01111110,
  0b01111110
};

uint8_t bar7[] = {
  0b11111110,
  0b11111110,
  0b11111110,
  0b11111110,
  0b11111110
};

void setup() {
  // put your setup code here, to run once:
  Serial.begin(9600);
  _delay_ms(500);
  vfd.begin(128, 32);
  vfd.interface(interface);
  vfd.isModelClass(903);
  vfd.isGeneration('S');
  vfd.GUD900_reset();
  vfd.GUD900_init();
  vfd.GUD900_useCustomChars(true);
  vfd.GUD900_defineCustomChar(0x20, GUD9005x7Format, bar1);
  vfd.GUD900_defineCustomChar(0x21, GUD9005x7Format, bar2);
  vfd.GUD900_defineCustomChar(0x22, GUD9005x7Format, bar3);
  vfd.GUD900_defineCustomChar(0x23, GUD9005x7Format, bar4);
  vfd.GUD900_defineCustomChar(0x24, GUD9005x7Format, bar5);
  vfd.GUD900_defineCustomChar(0x25, GUD9005x7Format, bar6);
  vfd.GUD900_defineCustomChar(0x26, GUD9005x7Format, bar7);
  vfd.GUD900_defineCustomChar(0x27, GUD9005x7Format, bar0);
}

void loop() {
  
  if(count == 0 && switch_position == 15){
    Serial.println("Read Configured");
    vfd.GUD900_touch_level_read();
  }
  interface.requestFrom(0x50, 20, true);
  while(interface.available() > 0){
      touchData = interface.read();
      Serial.println(touchData);
      if(touchData >= 0){
          if(touchData == TOUCH_STATUS_IDENTIFIER){
            count = 1;
          }else if(count == 1 && touchData == 16){
            switchNum = touchData;
            count = 2;
          }else if(count == 2){
            actMemory[switch_position] = touchData;
            switch_position--;
          }
          if(switch_position < 0){
            count = 0;
            touchData = 0;
            switch_position = 15;
          }
      }
  }
  updateSections();
}

void updateSections(){
  offset = 0;
  for(int i = 0; i < 8; i++){
    vfd.GUD900_setCursor(0 + offset, 8);
    updateBars(i, 0);
    offset += 16;
  }
  offset = 0;
  for(int i = 8; i < 16; i++){
    vfd.GUD900_setCursor(0 + offset, 24);
    updateBars(i, 16);
    offset += 16;
  }
}

void updateBars(int key, int topPos){
  switch(actMemory[key]){
    case 0:
      vfd.print((char)0x27);
      vfd.print((char)0x27);
      vfd.GUD900_setCursor(0 + offset, topPos);
      vfd.print((char)0x27);
      vfd.print((char)0x27);
      break;
    case 1:
      vfd.print((char)0x27);
      vfd.print((char)0x27);
      vfd.GUD900_setCursor(0 + offset, topPos);
      vfd.print((char)0x27);
      vfd.print((char)0x27);
      break;
    case 2:
      vfd.print((char)0x20);
      vfd.print((char)0x20);
      vfd.GUD900_setCursor(0 + offset, topPos);
      vfd.print((char)0x27);
      vfd.print((char)0x27);
      break;
    case 3:
      vfd.print((char)0x21);
      vfd.print((char)0x21);
      vfd.GUD900_setCursor(0 + offset, topPos);
      vfd.print((char)0x27);
      vfd.print((char)0x27);
      break;
    case 4:
      vfd.print((char)0x22);
      vfd.print((char)0x22);
      vfd.GUD900_setCursor(0 + offset, topPos);
      vfd.print((char)0x27);
      vfd.print((char)0x27);
      break;
    case 5:
      vfd.print((char)0x23);
      vfd.print((char)0x23);
      vfd.GUD900_setCursor(0 + offset, topPos);
      vfd.print((char)0x27);
      vfd.print((char)0x27);
      break;
    case 6:
      vfd.print((char)0x24);
      vfd.print((char)0x24);
      vfd.GUD900_setCursor(0 + offset, topPos);
      vfd.print((char)0x27);
      vfd.print((char)0x27);
      break;
    case 7:
      vfd.print((char)0x25);
      vfd.print((char)0x25);
      vfd.GUD900_setCursor(0 + offset, topPos);
      vfd.print((char)0x27);
      vfd.print((char)0x27);
      break;
    case 8:
      vfd.print((char)0x26);
      vfd.print((char)0x26);
      vfd.GUD900_setCursor(0 + offset, topPos);
      vfd.print((char)0x20);
      vfd.print((char)0x20);
      break;
    case 9:
      vfd.print((char)0x26);
      vfd.print((char)0x26);
      vfd.GUD900_setCursor(0 + offset, topPos);
      vfd.print((char)0x21);
      vfd.print((char)0x21);
      break;
    case 10:
      vfd.print((char)0x26);
      vfd.print((char)0x26);
      vfd.GUD900_setCursor(0 + offset, topPos);
      vfd.print((char)0x22);
      vfd.print((char)0x22);
      break;
    case 11:
      vfd.print((char)0x26);
      vfd.print((char)0x26);
      vfd.GUD900_setCursor(0 + offset, topPos);
      vfd.print((char)0x23);
      vfd.print((char)0x23);
      break;
    case 12:
      vfd.print((char)0x26);
      vfd.print((char)0x26);
      vfd.GUD900_setCursor(0 + offset, topPos);
      vfd.print((char)0x24);
      vfd.print((char)0x24);
      break;
    case 13:
      vfd.print((char)0x26);
      vfd.print((char)0x26);
      vfd.GUD900_setCursor(0 + offset, topPos);
      vfd.print((char)0x25);
      vfd.print((char)0x25);
      break;
    case 14:
      vfd.print((char)0x26);
      vfd.print((char)0x26);
      vfd.GUD900_setCursor(0 + offset, topPos);
      vfd.print((char)0x26);
      vfd.print((char)0x26);
      break;
    case 15:
      vfd.print((char)0x26);
      vfd.print((char)0x26);
      vfd.GUD900_setCursor(0 + offset, topPos);
      vfd.print((char)0x26);
      vfd.print((char)0x26);
      break;
  }
}

/*
if(actMemory[i] == 0){
  vfd.print(" ");
}else if(actMemory[i] == 1){
  vfd.print((char)0x20);
}else if(actMemory[i] == 2){
  vfd.print((char)0x21);
}else if(actMemory[i] == 3){
  vfd.print((char)0x22);
}else if(actMemory[i] == 4){
  vfd.print((char)0x23);
}else if(actMemory[i] == 5){
  vfd.print((char)0x24);
}else if(actMemory[i] == 6){
  vfd.print((char)0x25);
}else if(actMemory[i] == 7){
  vfd.print((char)0x26);
}else if(actMemory[i] == 8){
  vfd.GUD900_setCursor(0 + offset, 0);
  vfd.print((char)0x20);
}else if(actMemory[i] == 9){
  vfd.GUD900_setCursor(0 + offset, 0);
  vfd.print((char)0x21);
}else if(actMemory[i] == 10){
  vfd.GUD900_setCursor(0 + offset, 0);
  vfd.print((char)0x22);
}else if(actMemory[i] == 11){
  vfd.GUD900_setCursor(0 + offset, 0);
  vfd.print((char)0x23);
}else if(actMemory[i] == 12){
  vfd.GUD900_setCursor(0 + offset, 0);
  vfd.print((char)0x24);
}else if(actMemory[i] == 13){
  vfd.GUD900_setCursor(0 + offset, 0);
  vfd.print((char)0x25);
}else if(actMemory[i] == 14 || actMemory[i] == 15){
  vfd.GUD900_setCursor(0 + offset, 0);
  vfd.print((char)0x26);
}

if(actMemory[i] >= 0 && actMemory[i] <= 3){
      vfd.print((char)0x20);
      vfd.print((char)0x20);
    }else if(actMemory[i] > 3 && actMemory[i] <= 5){
      vfd.print((char)0x21);
      vfd.print((char)0x21);
    }else if(actMemory[i] > 5 && actMemory[i] <= 7){
      vfd.print((char)0x22);
      vfd.print((char)0x22);
    }else if(actMemory[i] > 7 && actMemory[i] <= 9){
      vfd.print((char)0x23);
      vfd.print((char)0x23);
    }else if(actMemory[i] > 9 && actMemory[i] <= 11){
      vfd.print((char)0x24);
      vfd.print((char)0x24);
    }else if(actMemory[i] > 11 && actMemory[i] <= 13){
      vfd.print((char)0x25);
      vfd.print((char)0x25);
    }else if(actMemory[i] > 13 && actMemory[i] <= 15){
      vfd.print((char)0x26);
      vfd.print((char)0x26);
    }else{
      vfd.print(" ");
    }

if(actMemory[i] == 0){
      vfd.print((char)0x27);
      vfd.print((char)0x27);
      vfd.GUD900_setCursor(0 + offset, 0);
      vfd.print((char)0x27);
      vfd.print((char)0x27);
    }else if(actMemory[i] == 1){
      vfd.print((char)0x27);
      vfd.print((char)0x27);
      vfd.GUD900_setCursor(0 + offset, 0);
      vfd.print((char)0x27);
      vfd.print((char)0x27);
    }else if(actMemory[i] == 2){
      vfd.print((char)0x20);
      vfd.print((char)0x20);
      vfd.GUD900_setCursor(0 + offset, 0);
      vfd.print((char)0x27);
      vfd.print((char)0x27);
    }else if(actMemory[i] == 3){
      vfd.print((char)0x21);
      vfd.print((char)0x21);
      vfd.GUD900_setCursor(0 + offset, 0);
      vfd.print((char)0x27);
      vfd.print((char)0x27);
    }else if(actMemory[i] == 4){
      vfd.print((char)0x22);
      vfd.print((char)0x22);
      vfd.GUD900_setCursor(0 + offset, 0);
      vfd.print((char)0x27);
      vfd.print((char)0x27);
    }else if(actMemory[i] == 5){
      vfd.print((char)0x23);
      vfd.print((char)0x23);
      vfd.GUD900_setCursor(0 + offset, 0);
      vfd.print((char)0x27);
      vfd.print((char)0x27);
    }else if(actMemory[i] == 6){
      vfd.print((char)0x24);
      vfd.print((char)0x24);
      vfd.GUD900_setCursor(0 + offset, 0);
      vfd.print((char)0x27);
      vfd.print((char)0x27);
    }else if(actMemory[i] == 7){
      vfd.print((char)0x25);
      vfd.print((char)0x25);
      vfd.GUD900_setCursor(0 + offset, 0);
      vfd.print((char)0x27);
      vfd.print((char)0x27);
    }else if(actMemory[i] == 8){
      vfd.print((char)0x26);
      vfd.print((char)0x26);
      vfd.GUD900_setCursor(0 + offset, 0);
      vfd.print((char)0x20);
      vfd.print((char)0x20);
    }else if(actMemory[i] == 9){
      vfd.print((char)0x26);
      vfd.print((char)0x26);
      vfd.GUD900_setCursor(0 + offset, 0);
      vfd.print((char)0x21);
      vfd.print((char)0x21);
    }else if(actMemory[i] == 10){
      vfd.print((char)0x26);
      vfd.print((char)0x26);
      vfd.GUD900_setCursor(0 + offset, 0);
      vfd.print((char)0x22);
      vfd.print((char)0x22);
    }else if(actMemory[i] == 11){
      vfd.print((char)0x26);
      vfd.print((char)0x26);
      vfd.GUD900_setCursor(0 + offset, 0);
      vfd.print((char)0x23);
      vfd.print((char)0x23);
    }else if(actMemory[i] == 12){
      vfd.print((char)0x26);
      vfd.print((char)0x26);
      vfd.GUD900_setCursor(0 + offset, 0);
      vfd.print((char)0x24);
      vfd.print((char)0x24);
    }else if(actMemory[i] == 13){
      vfd.print((char)0x26);
      vfd.print((char)0x26);
      vfd.GUD900_setCursor(0 + offset, 0);
      vfd.print((char)0x25);
      vfd.print((char)0x25);
    }else if(actMemory[i] == 14 || actMemory[i] == 15){
      vfd.print((char)0x26);
      vfd.print((char)0x26);
      vfd.GUD900_setCursor(0 + offset, 0);
      vfd.print((char)0x26);
      vfd.print((char)0x26);
    }
  switch(actMemory[i]){
    case 0:
      vfd.print((char)0x27);
      vfd.print((char)0x27);
      vfd.GUD900_setCursor(0 + offset, 0);
      vfd.print((char)0x27);
      vfd.print((char)0x27);
      break;
    case 1:
      vfd.print((char)0x27);
      vfd.print((char)0x27);
      vfd.GUD900_setCursor(0 + offset, 0);
      vfd.print((char)0x27);
      vfd.print((char)0x27);
      break;
    case 2:
      vfd.print((char)0x20);
      vfd.print((char)0x20);
      vfd.GUD900_setCursor(0 + offset, 0);
      vfd.print((char)0x27);
      vfd.print((char)0x27);
      break;
    case 3:
      vfd.print((char)0x21);
      vfd.print((char)0x21);
      vfd.GUD900_setCursor(0 + offset, 0);
      vfd.print((char)0x27);
      vfd.print((char)0x27);
      break;
    case 4:
      vfd.print((char)0x22);
      vfd.print((char)0x22);
      vfd.GUD900_setCursor(0 + offset, 0);
      vfd.print((char)0x27);
      vfd.print((char)0x27);
      break;
    case 5:
      vfd.print((char)0x23);
      vfd.print((char)0x23);
      vfd.GUD900_setCursor(0 + offset, 0);
      vfd.print((char)0x27);
      vfd.print((char)0x27);
      break;
    case 6:
      vfd.print((char)0x24);
      vfd.print((char)0x24);
      vfd.GUD900_setCursor(0 + offset, 0);
      vfd.print((char)0x27);
      vfd.print((char)0x27);
      break;
    case 7:
      vfd.print((char)0x25);
      vfd.print((char)0x25);
      vfd.GUD900_setCursor(0 + offset, 0);
      vfd.print((char)0x27);
      vfd.print((char)0x27);
      break;
    case 8:
      vfd.print((char)0x26);
      vfd.print((char)0x26);
      vfd.GUD900_setCursor(0 + offset, 0);
      vfd.print((char)0x20);
      vfd.print((char)0x20);
      break;
    case 9:
      vfd.print((char)0x26);
      vfd.print((char)0x26);
      vfd.GUD900_setCursor(0 + offset, 0);
      vfd.print((char)0x21);
      vfd.print((char)0x21);
      break;
    case 10:
      vfd.print((char)0x26);
      vfd.print((char)0x26);
      vfd.GUD900_setCursor(0 + offset, 0);
      vfd.print((char)0x22);
      vfd.print((char)0x22);
      break;
    case 11:
      vfd.print((char)0x26);
      vfd.print((char)0x26);
      vfd.GUD900_setCursor(0 + offset, 0);
      vfd.print((char)0x23);
      vfd.print((char)0x23);
      break;
    case 12:
      vfd.print((char)0x26);
      vfd.print((char)0x26);
      vfd.GUD900_setCursor(0 + offset, 0);
      vfd.print((char)0x24);
      vfd.print((char)0x24);
      break;
    case 13;
      vfd.print((char)0x26);
      vfd.print((char)0x26);
      vfd.GUD900_setCursor(0 + offset, 0);
      vfd.print((char)0x25);
      vfd.print((char)0x25);
      break;
    case 14:
      vfd.print((char)0x26);
      vfd.print((char)0x26);
      vfd.GUD900_setCursor(0 + offset, 0);
      vfd.print((char)0x26);
      vfd.print((char)0x26);
      break;
    case 15:
      vfd.print((char)0x26);
      vfd.print((char)0x26);
      vfd.GUD900_setCursor(0 + offset, 0);
      vfd.print((char)0x26);
      vfd.print((char)0x26);
      break;
  }
*/
